@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2023 -- leonerd@leonerd.org.uk
 
-package Text::Treesitter::Query 0.09;
+package Text::Treesitter::Query 0.10;
 
 use v5.14;
 use warnings;
@@ -126,8 +126,15 @@ where, in this case, the C<@name> capture had the capture index 123.
 
 =head2 test_predicates_for_match
 
+   $ok = $query->test_predicates_for_match( $match );
+
+An older form is also accepted:
+
    $ok = $query->test_predicates_for_match( $tree, $match );
 
+The I<$tree> argument is ignored if it is C<undef> or a
+C<Text::Treesitter::Tree> instance.
+
 Returns true if all the predicate tests in the given query match instance are
 successful (or if there are no predicates). Returns false if any predicate
 rejected it.
@@ -138,7 +145,7 @@ argument can be passed containing it, for efficient reuse and avoiding
 creating a second copy of the list.
 
    my @captures = $match->captures;
-   my $ok = $query->test_predicates_for_match( $tree, $match, \@captures );
+   my $ok = $query->test_predicates_for_match( $match, \@captures );
 
 The following predicate functions are recognised. Each also has an inverted
 variant whose name is preceeded by C<not-> to invert the logic.
@@ -252,8 +259,12 @@ sub test_predicate ( $self, $func, @args )
    }
 }
 
-sub test_predicates_for_match ( $self, $tree, $match, $_captures = undef )
+sub test_predicates_for_match
 {
+   my $self = shift;
+   shift if !defined $_[0] or $_[0]->isa( "Text::Treesitter::Tree" );
+   my ( $match, $_captures ) = @_;
+
    my @predicates = $self->predicates_for_pattern( $match->pattern_index ) 
       or return 1;
 
@@ -271,6 +282,10 @@ sub test_predicates_for_match ( $self, $tree, $match, $_captures = undef )
       my ( $func, @args ) = @$predicate;
       ref $_ and $_ = $captures_by_id{ $$_ } for @args;
 
+      # nvim extended the format to put directives as well as assertions; those
+      # have names ending in ! rather than ?
+      next unless $func =~ m/\?$/;
+
       $self->test_predicate( $func, @args ) or return 0;
    }
 
