@@ -4,12 +4,15 @@
 #  (C) Paul Evans, 2023 -- leonerd@leonerd.org.uk
 
 use v5.26;
+use warnings;
 use Object::Pad 0.73;
 
-package Text::Treesitter::QueryCursor 0.09;
+package Text::Treesitter::QueryCursor 0.10;
 class Text::Treesitter::QueryCursor
    :strict(params);
 
+use Carp;
+
 require Text::Treesitter::_XS;
 
 =head1 NAME
@@ -44,7 +47,8 @@ contain the matched patterns and captured nodes, which can then be queried.
 =cut
 
 field $querycursor;
-field $tree;
+field $_current_tree;
+field $_current_query;
 
 ADJUST
 {
@@ -84,7 +88,8 @@ method exec
    my ( $query, $node ) = @_;
 
    $querycursor->_exec( $query, $node->_node );
-   $tree = $node->tree;
+   $_current_tree = $node->tree;
+   $_current_query = $query;
 }
 
 =head2 next_match
@@ -102,7 +107,58 @@ method next_match
    my $querymatch = $querycursor->_next_match or return undef;
 
    require Text::Treesitter::QueryMatch;
-   return Text::Treesitter::QueryMatch->new( querymatch => $querymatch, tree => $tree );
+   return Text::Treesitter::QueryMatch->new( querymatch => $querymatch, tree => $_current_tree );
+}
+
+=head2 next_match_captures
+
+   $captures = $qc->next_match_captures( %options );
+
+I<Since version 0.10.>
+
+A convenience wrapper around L</next_match> for applying predicate tests and
+extracting the nodes corresponding to each capture.
+
+Matches that fail the C<test_predicates_for_match> are skipped. The next match
+that passes then has its captures extracted into a hash; with keys of the hash
+being the capture names, and the values containing the nodes.
+
+If the C<multi> option is true then each value of the returned hash will be an
+array reference containing every node that was captured at that name. If the
+option is false then it will contain just the final capture (which is normally
+fine because most patterns do not capture multiple nodes).
+
+=cut
+
+method next_match_captures ( %options )
+{
+   my $multi = delete $options{multi};
+   keys %options and
+      croak "Unrecognised options to ->next_captures: " . join( ", ", keys %options );
+
+   {
+      my $match = $self->next_match or return undef;
+
+      my @captures = $match->captures;
+
+      redo unless $_current_query->test_predicates_for_match( $match, \@captures );
+
+      my %captures_by_name;
+
+      foreach my $capture ( @captures ) {
+         my $node = $capture->node;
+         my $name = $_current_query->capture_name_for_id( $capture->capture_id );
+
+         if( $multi ) {
+            push $captures_by_name{ $name }->@*, $node;
+         }
+         else {
+            $captures_by_name{ $name } = $node;
+         }
+      }
+
+      return \%captures_by_name;
+   }
 }
 
 =head1 TODO
