@@ -4,9 +4,10 @@
 #  (C) Paul Evans, 2023 -- leonerd@leonerd.org.uk
 
 use v5.26;
+use warnings;
 use Object::Pad 0.70;
 
-package Text::Treesitter::QueryMatch 0.09;
+package Text::Treesitter::QueryMatch 0.10;
 class Text::Treesitter::QueryMatch
    :strict(params);
 
@@ -40,7 +41,7 @@ Usually accessed indirectly, via C<Text::Treesitter::QueryCursor>.
    while( my $match = $qc->next_match ) {
       my @captures = $match->captures;
 
-      next unless $query->test_predicates_for_match( $tree, $match, \@captures );
+      next unless $query->test_predicates_for_match( $match, \@captures );
 
       foreach my $capture ( @captures ) {
          my $node = $capture->node;
